@@ -77,11 +77,20 @@ seqGDS2VCF <- function(gdsfile, vcf.fn, info.var=NULL, fmt.var=NULL,
     outfmt <- 1L
     if (!inherits(vcf.fn, "connection"))
     {
-        ext <- substring(vcf.fn, nchar(vcf.fn)-2L)
-        if (ext == ".gz")
+        # get the file extension
+        pos <- regexpr("\\.([[:alnum:]]+)$", vcf.fn)
+        ext <- tolower(ifelse(pos > -1L, substring(vcf.fn, pos+1L), ""))
+        if (ext %in% c("gz", "bgz"))
         {
             if (.Platform$OS.type == "windows")
+            {
+                if (isTRUE(use_Rsamtools))
+                {
+                    warning("Rsamtools is not used on Windows.",
+                        immediate.=TRUE)
+                }
                 use_Rsamtools <- FALSE
+            }
             if (isTRUE(use_Rsamtools) && requireNamespace("Rsamtools"))
             {
                 ofile <- .Call(SEQ_bgzip_create, vcf.fn)
@@ -95,11 +104,11 @@ seqGDS2VCF <- function(gdsfile, vcf.fn, info.var=NULL, fmt.var=NULL,
                 ofile <- gzfile(vcf.fn, "wb")
                 outfmt <- 3L
             }
-        } else if (ext == ".bz")
+        } else if (ext == "bz")
         {
             ofile <- bzfile(vcf.fn, "wb")
             outfmt <- 4L
-        } else if (ext == ".xz")
+        } else if (ext == "xz")
         {
             ofile <- xzfile(vcf.fn, "wb")
             outfmt <- 5L
@@ -766,7 +775,8 @@ seqSNP2GDS <- function(gds.fn, out.fn, storage.option="LZMA_RA", major.ref=TRUE,
 
 seqBED2GDS <- function(bed.fn, fam.fn, bim.fn, out.gdsfn,
     compress.geno="LZMA_RA", compress.annotation="LZMA_RA", chr.conv=TRUE,
-    optimize=TRUE, digest=TRUE, parallel=FALSE, verbose=TRUE)
+    include.pheno=TRUE, optimize=TRUE, digest=TRUE, parallel=FALSE,
+    verbose=TRUE)
 {
     # check
     stopifnot(is.character(bed.fn), length(bed.fn)==1L)
@@ -784,6 +794,22 @@ seqBED2GDS <- function(bed.fn, fam.fn, bim.fn, out.gdsfn,
     stopifnot(is.character(compress.annotation), length(compress.annotation)==1L)
     stopifnot(is.logical(chr.conv), length(chr.conv)==1L)
     stopifnot(is.logical(optimize), length(optimize)==1L)
+    stopifnot(is.logical(include.pheno) | is.character(include.pheno))
+    nm_pheno <- c("family", "father", "mother", "sex", "phenotype")
+    if (is.logical(include.pheno))
+    {
+        if (!isTRUE(include.pheno) && !isFALSE(include.pheno))
+            stop("'include.pheno' should be TRUE, FALSE or a character vector.")
+        include.pheno <- if (include.pheno) nm_pheno else character()
+    }
+    if (is.character(include.pheno))
+    {
+        if (!all(include.pheno %in% nm_pheno))
+        {
+            stop("'include.pheno' should be one of ",
+                paste(nm_pheno, collapse=", "), ".")
+        }
+    }
     stopifnot(is.logical(digest) | is.character(digest), length(digest)==1L)
     stopifnot(is.logical(verbose), length(verbose)==1L)
     pnum <- .NumParallel(parallel)
@@ -804,13 +830,15 @@ seqBED2GDS <- function(bed.fn, fam.fn, bim.fn, out.gdsfn,
     bed_flag <- b[3L] == 0L
     if (verbose)
     {
-        cat("    bed file: ", sQuote(bed.fn), "\n        ",
+        cat("    BED file: ", sQuote(bed.fn), "\n        ",
             .pretty_size(file.size(bed.fn)), ", ",
             ifelse(bed_flag, "sample-major mode: [SNP, sample]",
             "SNP-major mode: [sample, SNP]"), "\n", sep="")
     }
 
     ##  read fam.fn  ##
+    if (verbose)
+        cat("    FAM file: ", sQuote(fam.fn), "\n", sep="")
     f <- .open_text(fam.fn, TRUE)
     famD <- read.table(f$con, header=FALSE, comment.char="",
         stringsAsFactors=FALSE)
@@ -822,17 +850,19 @@ seqBED2GDS <- function(bed.fn, fam.fn, bim.fn, out.gdsfn,
     } else {
         sample.id <- paste(famD$FamilyID, famD$InvID, sep="-")
         if (length(unique(sample.id)) != dim(famD)[1])
-            stop("IDs in PLINK BED are not unique!")
+            stop("Sample IDs in PLINK BED are not unique!")
     }
     if (verbose)
     {
         n <- nrow(famD)
-        cat("    fam file: ", sQuote(fam.fn), "\n        ",
+        cat("        ",
             .pretty_size(file.size(fam.fn)), ", ",
             .pretty(n), " sample", .plural(n), "\n", sep="")
     }
 
     ##  read bim.fn  ##
+    if (verbose)
+        cat("    BIM file: ", sQuote(bim.fn), "\n", sep="")
     f <- .open_text(bim.fn, TRUE)
     bimD <- read.table(f$con, header=FALSE, comment.char="",
         stringsAsFactors=FALSE)
@@ -841,7 +871,7 @@ seqBED2GDS <- function(bed.fn, fam.fn, bim.fn, out.gdsfn,
     if (verbose)
     {
         n <- nrow(bimD)
-        cat("    bim file: ", sQuote(bim.fn), "\n        ",
+        cat("        ",
             .pretty_size(file.size(bim.fn)), ", ",
             .pretty(n), " variant", .plural(n), "\n", sep="")
     }
@@ -880,36 +910,36 @@ seqBED2GDS <- function(bed.fn, fam.fn, bim.fn, out.gdsfn,
     put.attr.gdsn(n, "source.format", "PLINK BED Format")
 
     # add sample.id
-    if (verbose) cat("    sample.id")
+    if (verbose) cat("    sample.id  ")
     n <- add.gdsn(dstfile, "sample.id", sample.id, compress=compress.annotation,
         closezip=TRUE)
-    .DigestCode(n, digest, verbose)
+    .DigestCode(n, digest, verbose, FALSE)
 
     # add variant.id
-    if (verbose) cat("    variant.id")
+    if (verbose) cat("    variant.id  ")
     n <- add.gdsn(dstfile, "variant.id", seq_len(nrow(bimD)),
         compress=compress.annotation, closezip=TRUE)
-    .DigestCode(n, digest, verbose)
+    .DigestCode(n, digest, verbose, FALSE)
 
     # add position
-    if (verbose) cat("    position")
+    if (verbose) cat("    position  ")
     n <- add.gdsn(dstfile, "position", bimD$pos, storage="int32",
         compress=compress.annotation, closezip=TRUE)
-    .DigestCode(n, digest, verbose)
+    .DigestCode(n, digest, verbose, FALSE)
 
     # add chromosome
-    if (verbose) cat("    chromosome")
+    if (verbose) cat("    chromosome  ")
     n <- add.gdsn(dstfile, "chromosome", bimD$chr, storage="string",
         compress=compress.annotation, closezip=TRUE)
-    .DigestCode(n, digest, verbose)
+    .DigestCode(n, digest, verbose, FALSE)
     # RLE-coded chromosome
     .optim_chrom(dstfile)
 
     # add allele
-    if (verbose) cat("    allele")
+    if (verbose) cat("    allele  ")
     n <- add.gdsn(dstfile, "allele", paste(bimD$allele2, bimD$allele1, sep=","),
         storage="string", compress=compress.annotation, closezip=TRUE)
-    .DigestCode(n, digest, verbose)
+    .DigestCode(n, digest, verbose, FALSE)
 
     # add a folder for genotypes
     if (verbose)
@@ -1098,13 +1128,16 @@ seqBED2GDS <- function(bed.fn, fam.fn, bim.fn, out.gdsfn,
         }, split="none", tmp.fn=ptmpfn, nsamp=nrow(famD), psplit=psplit,
             cp=compress.annotation)
 
-        if (verbose) cat("        merging files ...")
-        lapply(ptmpfn, function(fn) {
+        if (verbose) cat("        merging files ... [")
+        lapply(seq_along(ptmpfn), function(i)
+        {
+            cat(ifelse(i>1L, ",", ""), i, sep="")
+            fn <- ptmpfn[i]
             f <- openfn.gds(fn)
             on.exit({ closefn.gds(f); unlink(fn, force=TRUE) })
             append.gdsn(n1, index.gdsn(f, "data"))
         })
-        if (verbose) cat(" [Done]\n      ")
+        if (verbose) cat(" Done]\n      ")
     }
     readmode.gdsn(n1)
     .DigestCode(n1, digest, verbose)
@@ -1152,26 +1185,45 @@ seqBED2GDS <- function(bed.fn, fam.fn, bim.fn, out.gdsfn,
     if (verbose) cat("    sample.annotation\n")
     n <- addfolder.gdsn(dstfile, "sample.annotation")
 
-    n1 <- add.gdsn(n, "family", famD$FamilyID, compress=compress.annotation,
-        closezip=TRUE)
-    .DigestCode(n1, digest, FALSE)
-
-    n1 <- add.gdsn(n, "father", famD$PatID, compress=compress.annotation,
-        closezip=TRUE)
-    .DigestCode(n1, digest, FALSE)
-
-    n1 <- add.gdsn(n, "mother", famD$MatID, compress=compress.annotation,
-        closezip=TRUE)
-    .DigestCode(n1, digest, FALSE)
-
-    sex <- rep("", length(sample.id))
-    sex[famD$Sex==1L] <- "M"; sex[famD$Sex==2L] <- "F"
-    n1 <- add.gdsn(n, "sex", sex, compress=compress.annotation, closezip=TRUE)
-    .DigestCode(n1, digest, FALSE)
-
-    n1 <- add.gdsn(n, "phenotype", famD$Pheno, compress=compress.annotation,
-        closezip=TRUE)
-    .DigestCode(n1, digest, FALSE)
+    if (length(include.pheno) && verbose) cat("       ")
+    if ("family" %in% include.pheno)
+    {
+        if (verbose) cat(" family")
+        n1 <- add.gdsn(n, "family", famD$FamilyID, compress=compress.annotation,
+            closezip=TRUE)
+        .DigestCode(n1, digest, FALSE)
+    }
+    if ("father" %in% include.pheno)
+    {
+        if (verbose) cat(" father")
+        n1 <- add.gdsn(n, "father", famD$PatID, compress=compress.annotation,
+            closezip=TRUE)
+        .DigestCode(n1, digest, FALSE)
+    }
+    if ("mother" %in% include.pheno)
+    {
+        if (verbose) cat(" mother")
+        n1 <- add.gdsn(n, "mother", famD$MatID, compress=compress.annotation,
+            closezip=TRUE)
+        .DigestCode(n1, digest, FALSE)
+    }
+    if ("sex" %in% include.pheno)
+    {
+        if (verbose) cat(" sex")
+        sex <- rep("", length(sample.id))
+        sex[famD$Sex==1L] <- "M"; sex[famD$Sex==2L] <- "F"
+        n1 <- add.gdsn(n, "sex", sex, compress=compress.annotation,
+            closezip=TRUE)
+        .DigestCode(n1, digest, FALSE)
+    }
+    if ("phenotype" %in% include.pheno)
+    {
+        if (verbose) cat(" phenotype")
+        n1 <- add.gdsn(n, "phenotype", famD$Pheno, compress=compress.annotation,
+            closezip=TRUE)
+        .DigestCode(n1, digest, FALSE)
+    }
+    if (length(include.pheno) && verbose) cat("\n")
 
     ##################################################
     # optimize access efficiency
